@@ -2,12 +2,12 @@
 ttyrpld - TTY replay daemon
 include/dev.h - Supply file for dev_t encoding
   Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2004
-  -- License restrictions apply (GPL2)
+  -- License restrictions apply (GPL v2)
 
   This file is part of ttyrpld.
   ttyrpld is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
-  the Free Software Foundation; however only version 2 of the License.
+  the Free Software Foundation; however ONLY version 2 of the License.
 
   ttyrpld is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,34 +19,29 @@ include/dev.h - Supply file for dev_t encoding
   Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.
 
-  -- For details see doc/GPL2.txt.
+  -- For details see the COPYING file
 =============================================================================*/
 #ifndef RPL_DEV_H
 #define RPL_DEV_H 1
 
-/* The Linux Kernel include files do not export these macros correctly,
-nor do the GLIBC include files provide correct and accurate ones. */
-
-#define K24_MINORBITS 8
-#define K24_MINORMASK ((1U << K24_MINORBITS) - 1)
-#define K24_MAJOR(dev) ((dev) >> K24_MINORBITS)
-#define K24_MINOR(dev) ((dev) & K24_MINORMASK)
-#define K24_MKDEV(major, minor) (((major) << K24_MINORBITS) | (minor))
-
-// Let's hope MINORBITS stays the same for some time :-)
-
 #define K26_MINORBITS 20
-#define K26_MINORMASK ((1U << K26_MINORBITS) - 1)
+#define K26_MINORMASK 0xFFFFF
 #define K26_MAJOR(dev) (unsigned long)((dev) >> K26_MINORBITS)
 #define K26_MINOR(dev) (unsigned long)((dev) & K26_MINORMASK)
-#define K26_MKDEV(major, minor) (((major) << K26_MINORBITS) | (minor))
+#define K26_MKDEV(major, minor) \
+    (((major) << K26_MINORBITS) | ((minor) & K26_MINORMASK))
 #define KD26_PARTS(dev) K26_MAJOR(dev), K26_MINOR(dev)
 
-// Glibc still uses MINORBITS = 8
+// Glibc uses an intermediate mixture to be 2.4-compatible
 
-#define GLIBC_MAJOR(dev) K24_MAJOR(dev)
-#define GLIBC_MINOR(dev) K24_MINOR(dev)
-#define GLIBC_MKDEV(major, minor) K24_MKDEV((major), (minor))
+#define COMPAT_MINORBITS 8
+#define COMPAT_MAJOR(dev) \
+    (unsigned long)(((dev) & 0xFFF) >> COMPAT_MINORBITS)
+#define COMPAT_MINOR(dev) \
+    (unsigned long)(((dev) & 0xFF) | (((dev) & 0xFFF) >> 20))
+#define COMPAT_MKDEV(major, minor) \
+    (((minor) & 0xFF) | (((minor) & 0xFFF00) << 20) | \
+    (((major) & 0xFFF) << COMPAT_MINORBITS))
 
 #endif // RPL_DEV_H
 
