@@ -2,12 +2,12 @@
 ttyrpld - TTY replay daemon
 kernel/rpldev.c - Kernel interface for RPLD
   Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2004
-  -- License restrictions apply (GPL2)
+  -- License restrictions apply (GPL v2)
 
   This file is part of ttyrpld.
   ttyrpld is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
-  the Free Software Foundation; however only version 2 of the License.
+  the Free Software Foundation; however ONLY version 2 of the License.
 
   ttyrpld is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,7 +19,7 @@ kernel/rpldev.c - Kernel interface for RPLD
   Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.
 
-  -- For details see doc/GPL2.txt.
+  -- For details see the COPYING file
 =============================================================================*/
 #include <asm/byteorder.h>
 #include <asm/semaphore.h>
@@ -30,6 +30,7 @@ kernel/rpldev.c - Kernel interface for RPLD
 #include <linux/kernel.h>
 #include <linux/miscdevice.h>
 #include <linux/module.h>
+#include <linux/moduleparam.h>
 #include <linux/poll.h>
 #include <linux/rpl.h>
 #include <linux/string.h>
@@ -44,28 +45,32 @@ kernel/rpldev.c - Kernel interface for RPLD
 #endif
 #if __GNUC__ == 3 && __GNUC_MINOR__ >= 4
      /* GCC 3.4 requires functions-to-be-inlined to appear before the call
-     to those. I'm not going that way. */
+     to those. Fleaky. Buggy? It's a GCC fault. It's not *my* duty to fix it.
+     At most to workaround. */
 #    define K_INLINE
 #else
 #    define K_INLINE inline
 #endif
 
-#if !(LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0) && \
-# LINUX_VERSION_CODE < KERNEL_VERSION(2, 7, 0))
-#    error *** Kernel versions other than 2.6.x are not supported
-#else
-#    define P_MAGIC MAGIC_2_6
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
+// should also work for earlier than 2.4, it's just for the device number thing
+#    define KERNEL_2_4 1
 #endif
 
 #ifndef KBUILD_BASENAME
 #    define KBUILD_BASENAME "rpldev"
 #endif
+#ifndef KBUILD_MODNAME
+#    define KBUILD_MODNAME "rpldev"
+#endif
 #define PREFIX __stringify(KBUILD_BASENAME) ": "
 
 #ifdef __LITTLE_ENDIAN
 #    define PP_swab(ptr)
-#else
+#elif defined(__BIG_ENDIAN)
 #    define PP_swab(ptr) my_swab((ptr), sizeof(*(ptr)))
+#else
+#    error *** Unsupported endianess
 #endif
 
 // Stage 2 functions
@@ -88,23 +93,23 @@ static unsigned int uif_poll(struct file *, poll_table *);
 static int uif_close(struct inode *, struct file *);
 
 // Local functions
-K_INLINE static ssize_t avail_R(void);
-K_INLINE static ssize_t avail_W(void);
-K_INLINE static unsigned int imin_uint(unsigned int, unsigned int);
-K_INLINE static void get_tty_devnr(uint32_t *, struct tty_struct *);
+static K_INLINE ssize_t avail_R(void);
+static K_INLINE ssize_t avail_W(void);
+static K_INLINE unsigned int imin_uint(unsigned int, unsigned int);
+static K_INLINE void get_tty_devnr(uint32_t *, struct tty_struct *);
+static K_INLINE unsigned long mkdev_26(unsigned long, unsigned long);
 static int mv_buffer(struct rpld_packet *, const void *, size_t);
-K_INLINE static void mv_buffer2(const void *, size_t);
-K_INLINE static void mv_to_user(char __user *, size_t);
-K_INLINE static void my_swab(void *, size_t);
+static K_INLINE void mv_buffer2(const void *, size_t);
+static K_INLINE void mv_to_user(char __user *, size_t);
+static K_INLINE void my_swab(void *, size_t);
 
 // Variables
 static DECLARE_WAIT_QUEUE_HEAD(Inqueue);
 static DECLARE_MUTEX(Open_lock);
 static DECLARE_MUTEX(Buffer_lock);
 static char *Buffer = NULL, *BufRP = NULL, *BufWP = NULL;
-// Each tty has a buffer of 4096, yet the number of possible ttys is endless
-static size_t Bufsize = 32768;
-static int Minor_nr = MISC_DYNAMIC_MINOR;
+static size_t Bufsize = 32768; // each tty has 4096, and 32K might not suffice
+static int Minor_nr = MISC_DYNAMIC_MINOR, WOverrun = 0;
 static struct file_operations Fops = {
     .open    = uif_open,
     .read    = uif_read,
@@ -118,15 +123,24 @@ static struct miscdevice Misc_info =
  {minor: MISC_DYNAMIC_MINOR, name: "rpl", fops: &Fops};
 
 // Module info
-int __init init_module(void);
-void __exit cleanup_module(void);
+extern int __init init_module(void);
+extern void __exit cleanup_module(void);
 MODULE_DESCRIPTION("ttyrpld Kernel interface");
 MODULE_AUTHOR("Jan Engelhardt <jengelh [at] linux01 gwdg de>");
 MODULE_LICENSE("GPL v2");
-MODULE_PARM(Bufsize, "i");
 MODULE_PARM_DESC(Bufsize, "Buffer size (default 32K)");
-MODULE_PARM(Minor_nr, "i");
 MODULE_PARM_DESC(Minor_nr, "Minor number to use (default: DYNAMIC)");
+MODULE_PARM_DESC(WOverrun, "Display Buffer Overrun messages (default: off)");
+#ifdef KERNEL_2_4
+MODULE_PARM(Bufsize, "i");
+MODULE_PARM(Minor_nr, "i");
+MODULE_PARM(WOverrun, "i");
+EXPORT_NO_SYMBOLS;
+#else
+module_param(Bufsize, uint, S_IRUGO);
+module_param(Minor_nr, int, S_IRUSR);
+module_param(WOverrun, bool, S_IRUGO | S_IWUSR);
+#endif
 
 //-----------------------------------------------------------------------------
 int __init init_module(void) {
@@ -136,7 +150,8 @@ int __init init_module(void) {
         cleanup_module();
         return eax;
     }
-    printk(PREFIX "Got registered at minor %d\n", Misc_info.minor);
+    Minor_nr = Misc_info.minor; // give it back to sysfs
+    printk(PREFIX "Got registered at minor %d\n", Minor_nr);
     return 0;
 }
 
@@ -152,7 +167,7 @@ static int krn_init(struct tty_struct *this, struct tty_struct *that) {
     if(this != NULL) { get_tty_devnr(&p.dev, this); }
     if(that != NULL) { get_tty_devnr(&p.dev2, that); }
     p.event = EVT_INIT;
-    p.magic = P_MAGIC;
+    p.magic = MAGIC_SIG;
     p.size  = 0;
     PP_swab(&p.dev);
     PP_swab(&p.dev2);
@@ -163,7 +178,7 @@ static int krn_open(struct tty_struct *this, struct tty_struct *that) {
     struct rpld_packet p;
     p.dev   = p.dev2 = 0;
     p.event = EVT_OPEN;
-    p.magic = P_MAGIC;
+    p.magic = MAGIC_SIG;
     p.size  = 0;
 
     get_tty_devnr(&p.dev, this);
@@ -182,7 +197,7 @@ static int krn_read(const char __user *buf, size_t count,
     struct rpld_packet p;
     p.dev2  = 0;
     p.event = EVT_READ;
-    p.magic = P_MAGIC;
+    p.magic = MAGIC_SIG;
     p.size  = count;
 
     get_tty_devnr(&p.dev, tty);
@@ -200,7 +215,7 @@ static int krn_write(const char __user *buf, size_t count,
     struct rpld_packet p;
     p.dev2  = 0;
     p.event = EVT_WRITE;
-    p.magic = P_MAGIC;
+    p.magic = MAGIC_SIG;
     p.size  = count;
 
     get_tty_devnr(&p.dev, tty);
@@ -221,7 +236,7 @@ static int krn_ioctl(struct tty_struct *tty, struct tty_struct *real_tty,
         PP_swab(&p.dev2);
     }
     p.event = EVT_IOCTL;
-    p.magic = P_MAGIC;
+    p.magic = MAGIC_SIG;
     p.size  = sizeof(cmd32);
     PP_swab(&p.dev);
     PP_swab(&p.size);
@@ -235,7 +250,7 @@ static int krn_close(struct tty_struct *this, struct tty_struct *that) {
     PP_swab(&p.dev);
     if(that != NULL) { get_tty_devnr(&p.dev2, that); PP_swab(&p.dev2); }
     p.event = EVT_CLOSE;
-    p.magic = P_MAGIC;
+    p.magic = MAGIC_SIG;
     p.size  = 0;
     return mv_buffer(&p, NULL, 0);
 }
@@ -246,7 +261,7 @@ static int krn_deinit(struct tty_struct *this, struct tty_struct *that) {
     if(this != NULL) { get_tty_devnr(&p.dev, this); }
     if(that != NULL) { get_tty_devnr(&p.dev2, that); }
     p.event = EVT_DEINIT;
-    p.magic = P_MAGIC;
+    p.magic = MAGIC_SIG;
     p.size  = 0;
     PP_swab(&p.dev);
     PP_swab(&p.dev2);
@@ -376,45 +391,55 @@ static int uif_close(struct inode *inode, struct file *filp) {
 }
 
 //-----------------------------------------------------------------------------
-K_INLINE static ssize_t avail_R(void) {
+static K_INLINE ssize_t avail_R(void) {
     // Return the number of available bytes to read
     if(BufWP >= BufRP) { return BufWP - BufRP; }
     return BufWP + Bufsize - BufRP;
 }
 
-K_INLINE static ssize_t avail_W(void) {
+static K_INLINE ssize_t avail_W(void) {
     // Return the number of available bytes to write
     if(BufWP >= BufRP) { return BufRP + Bufsize - BufWP - 1; }
     return BufRP - BufWP - 1;
 }
 
-K_INLINE static void get_tty_devnr(uint32_t *dev, struct tty_struct *tty) {
+static K_INLINE void get_tty_devnr(uint32_t *dev, struct tty_struct *tty) {
     // Get some unique number to identify the thing
-    *dev = MKDEV(tty->driver->major, tty->driver->minor_start + tty->index);
+#ifdef KERNEL_2_4
+    *dev = mkdev_26(MAJOR(tty->device), MINOR(tty->device));
+#else
+    *dev = mkdev_26(tty->driver->major, tty->driver->minor_start + tty->index);
+#endif
     return;
 }
 
-K_INLINE static unsigned int imin_uint(unsigned int a, unsigned int b) {
+static K_INLINE unsigned int imin_uint(unsigned int a, unsigned int b) {
     return a < b ? a : b;
 }
 
+static K_INLINE unsigned long mkdev_26(unsigned long maj, unsigned long min) {
+    return (maj << 20) | (min & 0xFFFFF);
+}
+
 static int mv_buffer(struct rpld_packet *p, const void *buf, size_t count) {
     /* Copy the contents the tty driver received to our circulary buffer and
     also add a header (rpld_packet) to it so that the userspace daemon can
     recognize the type. */
-    static int Overrun = 0;
+    static int has_overrun = 0;
 
     if(down_interruptible(&Buffer_lock)) { return -ERESTARTSYS; }
 
     if(avail_W() < sizeof(struct rpld_packet) + count) {
-        if(!Overrun) {
-            printk(KERN_INFO PREFIX "mv_buffer(): Buffer overrun\n");
-            ++Overrun;
+        if(!has_overrun) {
+            if(WOverrun) {
+                printk(KERN_INFO PREFIX "mv_buffer(): Buffer overrun\n");
+            }
+            ++has_overrun;;
         }
         up(&Buffer_lock);
         return -ENOSPC;
     } else {
-        Overrun = 0;
+        has_overrun = 0;
     }
 
     mv_buffer2(p, sizeof(struct rpld_packet));
@@ -424,7 +449,7 @@ static int mv_buffer(struct rpld_packet *p, const void *buf, size_t count) {
     return count;
 }
 
-K_INLINE static void mv_buffer2(const void *src, size_t count) {
+static K_INLINE void mv_buffer2(const void *src, size_t count) {
     /* This function is responsible for copying (a specific amount of
     arbitrary data) into the circulary buffer. (Taking the wrappage into
     account!) The parent function -- mv_buffer() in this case -- must make sure
@@ -443,7 +468,7 @@ K_INLINE static void mv_buffer2(const void *src, size_t count) {
     return;
 }
 
-K_INLINE static void mv_to_user(char __user *dest, size_t count) {
+static K_INLINE void mv_to_user(char __user *dest, size_t count) {
     /* mv_to_user() reads COUNT bytes from the circulary buffer and puts it
     into userspace memory. */
     int x = Buffer + Bufsize - BufRP;
@@ -462,7 +487,7 @@ K_INLINE static void mv_to_user(char __user *dest, size_t count) {
     return;
 }
 
-K_INLINE static void my_swab(void *srcp, size_t count) {
+static K_INLINE void my_swab(void *srcp, size_t count) {
     if(count == 2) {
         uint16_t *ptr = srcp;
         *ptr = cpu_to_le16(*ptr);
