@@ -2,7 +2,7 @@
 ttyrpld - TTY replay daemon
 user/replay.c - Realtime TTY log analyzer
   Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2004
-  -- License restrictions apply (GPL2)
+  -- License restrictions apply (GPL v2)
 
   This file is part of ttyrpld.
   ttyrpld is free software; you can redistribute it and/or modify it
@@ -19,7 +19,7 @@ user/replay.c - Realtime TTY log analyzer
   Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.
 
-  -- For details see doc/GPL2.txt.
+  -- For details see the COPYING file
 =============================================================================*/
 #include <sys/stat.h>
 #include <sys/time.h> // usec def, gettimeofday()
@@ -96,9 +96,8 @@ int main(int argc, const char **argv) {
     fprintf(stderr,
       "> ttyreplay " TTYRPLD_VERSION "\n"
       "This program comes with ABSOLUTELY NO WARRANTY; it is free software\n"
-      "and you are welcome to redistribute it under certain conditions;\n"
-      "for details see the doc/GPL2.txt file which should have come with\n"
-      "this program.\n\n"
+      "and you are welcome to redistribute it under certain conditions; for\n"
+      "details see the COPYING file which should have come with this program.\n"
     );
 
     if(argc == 0) {
