@@ -2,7 +2,7 @@
 ttyrpld - TTY replay daemon
 user/rplctl.c - INFOD client
   Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2004
-  -- License restrictions apply (GPL2)
+  -- License restrictions apply (GPL v2)
 
   This file is part of ttyrpld.
   ttyrpld is free software; you can redistribute it and/or modify it
@@ -19,7 +19,7 @@ user/rplctl.c - INFOD client
   Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.
 
-  -- For details see doc/GPL2.txt.
+  -- For details see the COPYING file
 =============================================================================*/
 #include <sys/select.h>
 #include <sys/socket.h>
@@ -82,8 +82,8 @@ static uint32_t getdev(const char *s) {
 
     if(s == NULL) { return 0; }
     if(*s == '=') {
-        /* The user can specify a major-minor number combination in the form of
-        "=MAJOR,MINOR" (BNF: "=" MAJOR "," MINOR)... */
+        /* The user can specify a major-minor number combination
+        in form of "=MAJOR,MINOR". */
         char *sdup = HX_strdup(s), *p = sdup, *minor_str;
         if((minor_str = strpbrk(p + 1, ",.:")) == NULL || p + 1 == minor_str) {
             fprintf(stderr, "getdev(): Incorrect device number specification"
@@ -117,9 +117,10 @@ static uint32_t getdev(const char *s) {
             return 0;
         }
 
-        major = GLIBC_MAJOR(sb.st_rdev);
-        minor = GLIBC_MINOR(sb.st_rdev);
+        major = COMPAT_MAJOR(sb.st_rdev);
+        minor = COMPAT_MINOR(sb.st_rdev);
     }
+
     return K26_MKDEV(major, minor);
 }
 
