@@ -2,7 +2,7 @@
 ttyrpld - TTY replay daemon
 user/rpld.c - User space daemon (filtering, etc.)
   Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2004
-  -- License restrictions apply (GPL2)
+  -- License restrictions apply (GPL v2)
 
   This file is part of ttyrpld.
   ttyrpld is free software; you can redistribute it and/or modify it
@@ -19,7 +19,7 @@ user/rpld.c - User space daemon (filtering, etc.)
   Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.
 
-  -- For details see doc/GPL2.txt.
+  -- For details see the COPYING file
 =============================================================================*/
 #include <sys/mman.h> // mlock()
 #include <sys/stat.h>
@@ -47,6 +47,7 @@ extern int posix_memalign(void **, size_t, size_t);
 #include "rpl_packet.h"
 #include "global.h"
 #include "rdsh.h"
+#include "stat2.h"
 #define NODEPATH "/var/run/.rpldev"
 
 // The prototype order is the same as the function body order
@@ -100,7 +101,7 @@ int main(int argc, const char **argv) {
     pthread_t infod_id;
     int fd;
 
-    umask(umask(0022) | 0477);
+    umask(umask(S_IWUSR | S_IWOTH) | S_IRUGO | S_IRWXG | S_IRWXO); // 0022,0477
 
     /* Yep, the config file is what is needed by all three
     (rpld, infod, rplctl). */
@@ -123,11 +124,9 @@ int main(int argc, const char **argv) {
     if(GOpt.verbose) {
         printf(
           "# rpld " TTYRPLD_VERSION "\n"
-          "This program comes with ABSOLUTELY NO WARRANTY; it is free"
-          " software\n"
-          "and you are welcome to redistribute it under certain conditions;\n"
-          "for details see the doc/GPL2.txt file which should have come with\n"
-          "this program.\n\n"
+          "This program comes with ABSOLUTELY NO WARRANTY; it is free software\n"
+          "and you are welcome to redistribute it under certain conditions; for\n"
+          "details see the COPYING file which should have come with this program.\n"
         );
     }
 
@@ -181,7 +180,7 @@ static void mainloop(int fd) {
         //PP_swab(&packet.dev2, sizeof(packet.dev2)); // .dev2 unused ATM
         PP_swab(&packet.size, sizeof(packet.size));
 
-        if(packet.magic != MAGIC_2_6) {
+        if(packet.magic != MAGIC_SIG) {
             ++Stats.badpack;
             notify(LOG_WARNING, "Bogus packet (magic is 0x%02X)!\n",
              packet.magic);
@@ -267,24 +266,6 @@ static int packet_preprox(struct mem_packet *packet) {
                 return 0;
             }
 
-            /* What's up with the ptm/pts stuffage:
-            Opening an xterm / screen window (with some debug printf in rpld):
-              EVT_INIT   dev=128:5 dev2=136:5 size=0
-              EVT_OPEN   dev=136:5 dev2=136:5 size=0
-
-            Closing it:
-              EVT_CLOSE  dev=136:5 dev2=128:5 size=0
-              EVT_CLOSE  dev=128:5 dev2=136:5 size=0
-              EVT_DEINIT dev=128:5 dev2=136:5 size=0
-
-            But killing the screen using ^A^K:
-              EVT_CLOSE  dev=128:5 dev2=136:5 size=0
-              EVT_CLOSE  dev=136:5 dev2=128:5 size=0
-              EVT_DEINIT dev=136:5 dev2=128:5 size=0
-
-            Not all CLOSEs and DEINITs are synchronous. Swapping the devs so
-            that dev always is 136: seems to be a good idea. And it works. */
-
             uint32_t tmp = packet->dev2;
             packet->dev2 = packet->dev;
             packet->dev  = tmp;
@@ -391,7 +372,7 @@ static int log_open(struct tty *tty) {
         notify(LOG_ERR, "Directory permission denied: It won't be possible to"
          " write to the file %s, expect warnings.\n", buf);
     }
-    if((tty->fd = open(buf, O_WRONLY | O_CREAT | O_APPEND, 0200)) < 0) {
+    if((tty->fd = open(buf, O_WRONLY | O_CREAT | O_APPEND, S_IWUSR)) < 0) {
         notify(LOG_ERR, "Could not open %s: %s\n", buf, strerror(errno));
     }
 
@@ -511,33 +492,41 @@ static const char *mknod_rpldev(void) {
     char buf[128];
     FILE *fp;
 
-    if((fp = fopen("/proc/misc", "r")) == NULL) {
-        perror("Could not open /proc/misc");
+    if((fp = fopen("/sys/module/rpldev/Minor_nr", "r")) != NULL) {
+        fgets(buf, sizeof(buf), fp);
+        minor = strtol(buf, NULL, 0);
+    } else if((fp = fopen("/proc/misc", "r")) == NULL) {
+        while(fgets(buf, sizeof(buf), fp) != NULL) {
+            char *ptr = buf, *name;
+            while(!isdigit(*ptr)) { ++ptr; }
+            minor = strtol(ptr, NULL, 0);
+            while(isdigit(*ptr)) { ++ptr; }
+            while(isspace(*ptr)) { ++ptr; }
+
+            name = ptr;
+            while(isalnum(*ptr)) { ++ptr; }
+            *ptr = '\0';
+            if(strcmp(name, "rpl") == 0) { break; }
+            minor = -1;
+        }
+    } else {
+        fprintf(stderr, "Could not open either /sys/module/rpldev/Minor_nr "
+         "or /proc/misc!\n");
+        fclose(fp);
         return NULL;
     }
 
-    while(fgets(buf, sizeof(buf), fp) != NULL) {
-        char *ptr = buf, *name;
-        while(!isdigit(*ptr)) { ++ptr; }
-        minor = strtol(ptr, NULL, 0);
-        while(isdigit(*ptr)) { ++ptr; }
-        while(isspace(*ptr)) { ++ptr; }
-
-        name = ptr;
-        while(isalnum(*ptr)) { ++ptr; }
-        *ptr = '\0';
-        if(strcmp(name, "rpl") == 0) { break; }
-        minor = -1;
-    }
-
     fclose(fp);
 
-    if(minor == -1) {
-        printf("RPL module does not seem to be loaded\n");
+    if(minor < 0) {
+        printf("rpldev module does not seem to be loaded or can not find\n"
+         "minor number associated with it.\n");
         return NULL;
     }
 
-    if(mknod(NODEPATH, S_IFCHR | 0400, GLIBC_MKDEV(10, minor)) != 0) {
+    unlink(NODEPATH); // safe side in case mknod() returns EEXIST
+    if(mknod(NODEPATH, S_IFCHR | S_IRUSR, COMPAT_MKDEV(10, minor)) != 0) {
+        perror("mknod()");
         return NULL;
     }
 
